@@ -22,6 +22,7 @@ from ywh2bt.core.api.models.report import (
     CommentLog,
     CvssUpdateLog,
     DetailsUpdateLog,
+    FixVerifiedLog,
     Log,
     PriorityUpdateLog,
     REPORT_STATUS_TRANSLATIONS,
@@ -377,13 +378,13 @@ class ReportSynchronizer:
         )
         is_created_issue = False
         tracker_issue = None
-        if log and isinstance(log, TrackingStatusLog):
+        if isinstance(log, TrackingStatusLog):
             is_created_issue = True
             tracker_issue = self._get_tracker_issue_from_logs(
                 log=log,
             )
         if tracker_issue is None:
-            if not self._synchronize_options.recreate_missing_issues:
+            if isinstance(log, TrackingStatusLog) and not self._synchronize_options.recreate_missing_issues:
                 return SynchronizeReportResult(
                     is_created_issue=is_created_issue,
                     is_existing_issue=False,
@@ -559,7 +560,7 @@ class ReportSynchronizer:
             isinstance(log, DetailsUpdateLog) and synchronize_options.upload_details_updates,
             isinstance(log, PriorityUpdateLog) and synchronize_options.upload_priority_updates,
             isinstance(log, RewardLog) and synchronize_options.upload_rewards,
-            isinstance(log, StatusUpdateLog) and synchronize_options.upload_status_updates,
+            isinstance(log, (StatusUpdateLog, FixVerifiedLog)) and synchronize_options.upload_status_updates,
         ))
 
     def _send_logs(
